﻿using System;
using System.Windows.Controls;
using System.Windows.Media;
using Stun2DPhysics4SL;
using Stun2DPhysics4SL.Mathematics;
using System.Windows;
using System.Collections.Generic;
using System.Windows.Input;

namespace Stun2DPhysics4SLDemo
{
    public partial class MainPage : UserControl
    {
        // 当一帧绘制结束时保存当前时刻
        private DateTime LastTick;
        // 物理引擎
        PhysicsSimulator physicsSimulator;

        // 保存Sprite的集合
        private List<Sprite> sprites;

        Box1Sprite box1;
        Box2Sprite box2;
        
        bool isRunning = false ;

        public MainPage()
        {
            InitializeComponent();

            physicsSimulator = new PhysicsSimulator();
            physicsSimulator.Gravity = new Vector2(0, 150);

            sprites = new List<Sprite>();

            box1 = new Box1Sprite(LayoutRoot, new Point(16, 16), physicsSimulator);
            box1.Body.LinearVelocity = new Vector2(150, 0);

            box2= new Box2Sprite(LayoutRoot, new Point(32, 448), physicsSimulator);
            box2.Body.IgnoreGravity = true;

            sprites.Add(box1);
            sprites.Add(box2);
        }
        
        private void CompositionTarget_Rendering(object sender, EventArgs e)
        {
            // 保存自上一次更新以来流逝的时间
            TimeSpan elapsedTime = (DateTime.Now - LastTick);

            physicsSimulator.Update((float)elapsedTime.TotalSeconds);
            
            // 对box1施加一个水平向右的风力
            const float forceAmount = 50;
            Vector2 force = Vector2.Zero;
            force += new Vector2(forceAmount, 0);
            box1.Body.ApplyForce(force);

            for (int i = 0; i < sprites.Count; i++)
            {
                sprites[i].Update();
            }

            // 处理物体在边界上的碰撞，以后会通过引擎中的碰撞检测实现
            if (box1.Body.Position.X > 784 || box1.Body.Position.X < 16)
                box1.Body.LinearVelocity.X *= -1;
            if (box1.Body.Position.Y > 464 || box1.Body.Position.Y < 16)
                box1.Body.LinearVelocity.Y *= -1;
            if (box2.Body.Position.X > 768 || box2.Body.Position.X < 32)
            {
                box2.Body.LinearVelocity.X *= -1;
                box2.Body.AngularVelocity *= -1;
            }
            if (box2.Body.Position.Y > 448 || box2.Body.Position.Y < 32)
            {
                box2.Body.LinearVelocity.Y *= -1;
                box2.Body.AngularVelocity *= -1;
            }

            // 保存当前时刻
            LastTick = DateTime.Now;
        }

        private void btnStart_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            if (!isRunning)
            {
                btnStart.Content = "暂停";
                // 保存当前时刻
                LastTick = DateTime.Now;
                CompositionTarget.Rendering += new EventHandler(CompositionTarget_Rendering);
                this.KeyDown += new KeyEventHandler(Page_KeyDown);
                isRunning = true;
            }
            else
            {
                btnStart.Content = "继续";
                CompositionTarget.Rendering -= new EventHandler(CompositionTarget_Rendering);
                this.KeyDown -= new KeyEventHandler(Page_KeyDown);
                isRunning = false;
            }
        }

        private void Page_KeyDown(object sender, KeyEventArgs e)
        {
            // 如果按下A键则对box2施加一个斜向上的冲量
            if(e.Key==Key.A)
            {
                box2.Body.ApplyLinearImpulse (new Vector2(50,-75));
                box2.Body.AngularVelocity = 3.0f;
                box2.Body.IgnoreGravity = false;
            }
        }
    }
}
